package de.hsel.itech.db;

import de.hsel.itech.db.pojo.Author;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 **/
public class AuthorDB {

    private Database database;


    AuthorDB(Database database) {
        this.database = database;
    }

    /**
     * gets tableAuthor by id.
     *
     * @param id ID from database
     * @return {@link de.hsel.itech.db.pojo.Author}
     */
    @Nullable
    public Author get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableAuthor, id);
        assert entry != null;

        ResultSet rs = entry.getKey();
        Author author = null;
        try {
            rs.beforeFirst();
            while(rs.next()) {
                author = new Author(rs.getLong("id"), rs.getString("name"));
            }
            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return author;
    }

    /**
     * gets tableAuthor by name.
     *
     * @param name tableAuthor name
     * @return {@link de.hsel.itech.db.pojo.Author}
     */
    @Nullable
    public Author get(@NotNull String name) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsByValue(database.tableAuthor, "name", name);
        assert entry != null;
        assert entry.getKey() != null;

        Author author = null;
        ResultSet rs = entry.getKey();
        try {
            if (rs.next())
                author = new Author(rs.getLong("id"), rs.getString("name"));
            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return author;
    }

    /**
     * inserts Author.
     *
     * @param author Author
     * @return insert count
     */
    public int insert(@NotNull Author author) {
        Connection connection = database.getConnection();
        assert connection != null;

        int insertCount = 0;
        try {
            PreparedStatement statement = connection.prepareStatement("INSERT INTO author (name) VALUES (?)");
            statement.setString(1, author.getName());

            insertCount += statement.executeUpdate();

            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return insertCount;
    }

    /**
     * deletes tableAuthor from database.
     *
     * @param id tableAuthor id
     * @return deletion count
     */
    public int delete(long id) {
        return database.delete(id, database.tableAuthor);
    }

    /**
     * gets list of tableAuthor ids from database.
     *
     * @param id tableBook id from database
     * @return {@link java.util.List}
     */
    @Nullable
    List<Integer> getAll(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsById(database.tableAuthorBook, database.tableBook, id);
        assert entry != null;
        try {
            List<Integer> list = new ArrayList<>();
            ResultSet rs = entry.getKey();
            while (rs.next()) {
                list.add(rs.getInt(database.tableAuthor));
            }
            entry.getValue().close();
            return list;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }
}