package de.hsel.itech.db;

import de.hsel.itech.db.pojo.Category;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 **/
public class CategoryDB {

    private Database database;

    CategoryDB(Database database) {
        this.database = database;
    }

    /**
     * gets tableCategory by id.
     *
     * @param id ID from database
     * @return {@link de.hsel.itech.db.pojo.Category}
     */
    @Nullable
    public Category get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableCategory, id);
        if(entry != null) {
            ResultSet rs = entry.getKey();

            try {
                rs.beforeFirst();
            } catch (SQLException e) {
                e.printStackTrace();
            }
            return get(entry);
        }
        return null;
    }

    /**
     * getAll tableCategory by name.
     *
     * @param name tableCategory name
     * @return {@link de.hsel.itech.db.pojo.Category}
     */
    @Nullable
    public Category get(@NotNull String name) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsByValue(database.tableCategory, "name", name);
        assert entry != null;

        return get(entry);
    }

    /**
     * getAll Category from database entry.
     * @param entry entry
     * @return Category
     */
    @Nullable
    private Category get(@NotNull Map.Entry<ResultSet, Connection> entry) {
        ResultSet rs = entry.getKey();
        Category category = null;
        try {
            while (rs.next()) {
                category = new Category(rs.getLong("id"), rs.getString("name"), rs.getString("image"));
            }
            entry.getValue().close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return category;
    }

    /**
     * getAll all categories.
     *
     * @return list of categories
     */
    @Nullable
    public List<Category> getAll() {
        List<Long> ids = database.getIds(database.tableCategory);
        List<Category> categories = new ArrayList<>();
        for (long id : ids) {
            categories.add(get(id));
        }
        return categories;
    }

    /**
     * deletes tableCategory from database.
     *
     * @param id tableCategory count
     * @return deletion count
     */
    public int delete(long id) {
        return database.delete(id, database.tableCategory);
    }

    /**
     * inserts Category into database.
     *
     * @param category Category
     * @return insert count
     */
    public int insert(@NotNull Category category) {
        Connection connection = database.getConnection();
        assert connection != null;

        int insertCount = 0;

        try {
            PreparedStatement statement = connection.prepareStatement("INSERT INTO category (name, image) VALUES (?, ?)");
            statement.setString(1, category.getName());
            statement.setString(2, category.getImage());
            insertCount += statement.executeUpdate();

            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return insertCount;
    }
}