package de.hsel.itech.db;

import de.hsel.itech.db.pojo.Publisher;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

/**
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 **/
public class PublisherDB {

    private Database database;

    PublisherDB(Database database) {
        this.database = database;
    }

    /**
     * gets tablePublisher by id.
     *
     * @param id ID from database
     * @return {@link de.hsel.itech.db.pojo.Publisher}
     */
    @Nullable
    public Publisher get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePublisher, id);
        assert entry != null;

        ResultSet rs = entry.getKey();
        Publisher publisher = null;
        try {
            rs.beforeFirst();
            while(rs.next()) {
                publisher = new Publisher(rs.getLong("id"), rs.getString("name"));
            }
            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return publisher;
    }

    /**
     * getAll tablePublisher by name.
     *
     * @param name tablePublisher name
     * @return {@link de.hsel.itech.db.pojo.Publisher}
     */
    @Nullable
    public Publisher get(@NotNull String name) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsByValue(database.tablePublisher, "name", name);
        assert entry != null;
        assert entry.getKey() != null;

        Publisher publisher = null;
        ResultSet rs = entry.getKey();
        try {
            while (rs.next()) {
                publisher = new Publisher(rs.getLong("id"), rs.getString("name"));
            }
            entry.getValue().close();
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return publisher;
    }




    /**
     * inserts Publisher.
     *
     * @param publisher Publisher
     * @return insert count
     */
    public int insert(@NotNull Publisher publisher) {
        Connection connection = database.getConnection();
        assert connection != null;

        int insertCount = 0;
        try {
            PreparedStatement statement = connection.prepareStatement("INSERT INTO publisher (name) VALUES (?)");
            statement.setString(1, publisher.getName());

            insertCount += statement.executeUpdate();

            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return insertCount;
    }

    /**
     * deletes tablePublisher from database.
     *
     * @param id tablePublisher id
     * @return deletion count
     */
    public int delete(long id) {
        return database.delete(id, database.tablePublisher);
    }
}