package de.hsel.itech.db;

import de.hsel.itech.db.pojo.User;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.mindrot.jbcrypt.BCrypt;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;

/**
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 **/
public class UserDB {

    private Database database;

    private final int logRounds = 11;

    UserDB(Database database) {
        this.database = database;
    }

    @Nullable
    public User get(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableUser, id);
        assert entry != null;

        ResultSet rs = entry.getKey();
        try {
            User user = null;
            while(rs.next()) {
                user =  new User(id, rs.getString("email"), rs.getString("name"), rs.getString("password"), rs.getShort("type"));
            }
            entry.getValue().close();
            return user;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    public User get(String name) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetsByValue(database.tableUser, "name", name);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            User user = null;
            while(rs.next()) {
                user =  new User(rs.getLong("id"), rs.getString("email"), rs.getString("name"), rs.getString("password"), rs.getShort("type"));
            }
            entry.getValue().close();
            return user;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    public boolean verify(@NotNull User user, @NotNull String password) {
        return verifyHash(password, user.getPassword());
    }


    public int insert(@NotNull User user) {
        user.setPassword(hash(user.getPassword()));


        Connection connection = database.getConnection();
        int updateCount = 0;

        try {
            assert connection != null;
            PreparedStatement statement = connection.prepareStatement("INSERT INTO user (name, email, password, type) values (?, ?, ?, ?)");
            statement.setString(1, user.getName());
            statement.setString(2, user.getEmail());
            statement.setString(3, user.getPassword());
            statement.setShort(4, user.getType());

            updateCount += statement.executeUpdate();
            connection.close();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return updateCount;
    }

    public int delete(@NotNull User user) {
        return database.delete(user.getId(), database.tableUser);
    }

    @NotNull
    private String hash(@NotNull String password) {
        return BCrypt.hashpw(password, BCrypt.gensalt(logRounds));
    }

    private boolean verifyHash(@NotNull String plaintext, @NotNull String hash) {
        return BCrypt.checkpw(plaintext, hash);
    }
}