package de.hsel.itech.servlet;

import de.hsel.itech.db.Database;
import de.hsel.itech.db.pojo.Author;
import de.hsel.itech.db.pojo.Book;
import de.hsel.itech.db.pojo.Category;
import de.hsel.itech.db.pojo.Publisher;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.charset.StandardCharsets;
import java.time.Year;
import java.util.ArrayList;
import java.util.List;

/**
 * Servlet: list of all books and remove function
 *
 * @author Julian Hinxlage
 * @version 0.1
 * @since 0.1
 */

@WebServlet("/booklist")
public class BookList extends HttpServlet {

    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
       doPost(req, resp);
    }

    @Override
    protected void doPost(@NotNull final HttpServletRequest req, @NotNull final HttpServletResponse resp) throws IOException {
        req.setCharacterEncoding(StandardCharsets.UTF_8.name());

        //getAll database object
        Database db = Database.getInstance();

        resp.setCharacterEncoding(StandardCharsets.UTF_8.name());
        PrintWriter out = resp.getWriter();

        //header template
        Utillity.insertFile(out, "template_head.html");

        //remove book by id
        String param = req.getParameter("removeid");
        if (param != null && Long.parseLong(param) != 0) {
            long isbn = Long.parseLong(param);
            db.book().delete(isbn);
            resp.sendRedirect("booklist");
        }

        //add book to database
        if (req.getParameter("title") != null) {
            try {

                long isbn = Long.parseLong(req.getParameter("isbn"));
                String title = req.getParameter("title");
                String authorName = req.getParameter("author");
                String priceString = req.getParameter("price");
                String[] priceParts = priceString.split(",");
                int price = Integer.parseInt(priceParts[0]) * 100;
                if (priceParts.length >= 2) {
                    if (priceParts[1].length() == 1) {
                        price += Integer.parseInt(priceParts[1]) * 10;
                    } else {
                        price += Integer.parseInt(priceParts[1]);
                    }
                }
                int year = Integer.parseInt(req.getParameter("year"));
                String categoryName = req.getParameter("category");
                String description = req.getParameter("description");
                String publisher = req.getParameter("publisher");
                String image = req.getParameter("image");

                //check isbn
                if (db.book().exists(isbn)) {
                    out.println("<aside class =\"m-note m-danger\">");
                    out.println("<h3>Es gibt bereits ein Buch mit dieser ISBN </h3>");
                    out.println("</aside>");
                } else {
                    ArrayList<Author> authors = new ArrayList<>();
                    String[] authorNames = authorName.split(",");
                    for (String name : authorNames) {
                        authors.add(new Author(name));
                    }
                    Category category = new Category(categoryName);
                    Book book = new Book(isbn,
                            authors,
                            new Publisher(publisher),
                            category,
                            title, Year.of(year), price, description, image
                    );
                    db.book().insert(book);
                    resp.sendRedirect("booklist");
                }
            } catch (Exception e) {
                e.printStackTrace();
                out.println("<aside class =\"m-note m-danger\">");
                out.println("<h3>Es gibt ein Fehler in der Eingabe </h3>");
                out.println("</aside>");
            }

        }


        //list all books
        List<Book> books = db.book().getAll();
        if (books != null) {

            //book entries
            out.println("<div class=\"m-container\">");
            for (Book book : books) {

                out.println(" <div class=\"m-row\">");

                out.println("  <div class=\"m-col-l-4\">");
                out.println("   <img class=\"m-image\" src=\"https://source.unsplash.com/" + book.getImage() + "/300x300\" alt=\"Buchcover\">");
                out.println("  </div>");

                out.println("  <div class=\"m-col-l-4\">");
                out.println("<h2>" + book.getTitle() + "</h2>");
                out.println("ISBN: " + book.getIsbn() + "</br>");

                List<String> authors = new ArrayList<>();
                for (Author author : book.getAuthors()) {
                    authors.add(author.getName());
                }

                out.println("von: " + String.join(", ", authors));

                out.println("</br>Veröffentlicht von " + book.getPublisher().getName() + "");
                out.println("</br></br>" + book.getDescription() + "");

                out.println("  </div>");

                out.println("  <div class=\"m-col-l-4\">");
                out.println("   <div class=\"m-button m-danger\">");

                out.println("<a href=\"/itech/booklist?removeid=" + book.getId() + "\">");
                out.println("Entfernen");
                out.println("</a>");

                out.println("   <br>");
                out.println("   <br>");

                out.print("   <h3>" + book.getPrice() / 100 + ",");
                if (book.getPrice() % 100 < 10) {
                    out.print("0");
                }

                out.println(book.getPrice() % 100 + " &euro;</h3>");

                out.println(" </div>");
                out.println("   </div>");
                out.println("  </div>");
            }
            out.println("</div>");
        }

        //add book button
        Utillity.addButton(out, "Neues Buch Hinzufügen", "addbook");

        //footer template
        Utillity.insertFile(out, "template_footer.html");
    }

}
